@@ -492,6 +492,21 @@ gst_byte_writer_put_data (GstByteWriter * writer, const guint8 * data,
   return TRUE;
 }
 
+gboolean
+gst_byte_writer_fill (GstByteWriter * writer, const guint8 value, guint size)
+{
+  g_return_val_if_fail (writer != NULL, FALSE);
+
+  if (G_UNLIKELY (!gst_byte_writer_ensure_free_space (writer, size)))
+    return FALSE;
+
+  memset ((guint8 *) & writer->parent.data[writer->parent.byte], value, size);
+  writer->parent.byte += size;
+  writer->parent.size = MAX (writer->parent.size, writer->parent.byte);
+
+  return TRUE;
+}
+
 #define CREATE_WRITE_STRING_FUNC(bits,type) \
 gboolean \
 gst_byte_writer_put_string_utf##bits (GstByteWriter *writer, const type * data) \
@@ -718,7 +733,7 @@ CREATE_WRITE_STRING_FUNC (32, guint32);
  * Since: 0.10.26
  */
 /**
- * gst_byte_writer_put_float_be:
+ * gst_byte_writer_put_float32_be:
  * @writer: #GstByteWriter instance
  * @val: Value to write
  *
@@ -729,7 +744,7 @@ CREATE_WRITE_STRING_FUNC (32, guint32);
  * Since: 0.10.26
  */
 /**
- * gst_byte_writer_put_double_be:
+ * gst_byte_writer_put_float64_be:
  * @writer: #GstByteWriter instance
  * @val: Value to write
  *
@@ -740,7 +755,7 @@ CREATE_WRITE_STRING_FUNC (32, guint32);
  * Since: 0.10.26
  */
 /**
- * gst_byte_writer_put_float_le:
+ * gst_byte_writer_put_float32_le:
  * @writer: #GstByteWriter instance
  * @val: Value to write
  *
@@ -751,7 +766,7 @@ CREATE_WRITE_STRING_FUNC (32, guint32);
  * Since: 0.10.26
  */
 /**
- * gst_byte_writer_put_double_le:
+ * gst_byte_writer_put_float64_le:
  * @writer: #GstByteWriter instance
  * @val: Value to write
  *
@@ -806,3 +821,15 @@ CREATE_WRITE_STRING_FUNC (32, guint32);
  *
  * Since: 0.10.26
  */
+/**
+ * gst_byte_writer_fill:
+ * @writer: #GstByteWriter instance
+ * @value: Value to be writen
+ * @size: Number of bytes to be writen
+ *
+ * Writes @size bytes containing @value to @writer.
+ *
+ * Returns: %TRUE if the value could be written
+ *
+ * Since: 0.10.26
+ */
