@@ -77,6 +77,15 @@
 GST_DEBUG_CATEGORY_STATIC (collect_pads_debug);
 #define GST_CAT_DEFAULT collect_pads_debug
 
+#define GST_COLLECT_PADS_GET_PRIVATE(obj)  \
+  (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GST_TYPE_COLLECT_PADS, GstCollectPadsPrivate))
+
+struct _GstCollectPadsPrivate
+{
+  GstCollectPadsClipFunction clipfunc;
+  gpointer clipfunc_user_data;
+};
+
 GST_BOILERPLATE (GstCollectPads, gst_collect_pads, GstObject, GST_TYPE_OBJECT);
 
 static void gst_collect_pads_clear (GstCollectPads * pads,
@@ -88,6 +97,7 @@ static void gst_collect_pads_init (GstCollectPads * pads,
     GstCollectPadsClass * g_class);
 static void ref_data (GstCollectData * data);
 static void unref_data (GstCollectData * data);
+static void gst_collect_pads_check_pads_unlocked (GstCollectPads * pads);
 
 static void
 gst_collect_pads_base_init (gpointer g_class)
@@ -100,15 +110,19 @@ gst_collect_pads_class_init (GstCollectPadsClass * klass)
 {
   GObjectClass *gobject_class = (GObjectClass *) klass;
 
+  g_type_class_add_private (klass, sizeof (GstCollectPadsPrivate));
+
   GST_DEBUG_CATEGORY_INIT (collect_pads_debug, "collectpads", 0,
       "GstCollectPads");
 
-  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_collect_pads_finalize);
+  gobject_class->finalize = gst_collect_pads_finalize;
 }
 
 static void
 gst_collect_pads_init (GstCollectPads * pads, GstCollectPadsClass * g_class)
 {
+  pads->abidata.ABI.priv = GST_COLLECT_PADS_GET_PRIVATE (pads);
+
   pads->cond = g_cond_new ();
   pads->data = NULL;
   pads->cookie = 0;
@@ -152,7 +166,7 @@ gst_collect_pads_finalize (GObject * object)
 /**
  * gst_collect_pads_new:
  *
- * Create a new instance of #GstCollectsPads.
+ * Create a new instance of #GstCollectPads.
  *
  * Returns: a new #GstCollectPads, or NULL in case of an error.
  *
@@ -163,7 +177,7 @@ gst_collect_pads_new (void)
 {
   GstCollectPads *newcoll;
 
-  newcoll = g_object_new (GST_TYPE_COLLECT_PADS, NULL);
+  newcoll = g_object_newv (GST_TYPE_COLLECT_PADS, 0, NULL);
 
   return newcoll;
 }
@@ -237,6 +251,10 @@ unref_data (GstCollectData * data)
  * gst_collect_pads_remove_pad() to remove the pad from the collection
  * again.
  *
+ * This function will override the chain and event functions of the pad
+ * along with the element_private data, which is used to store private
+ * information for the collectpads.
+ *
  * You specify a size for the returned #GstCollectData structure
  * so that you can use it to store additional information.
  *
@@ -342,6 +360,32 @@ find_pad (GstCollectData * data, GstPad * pad)
   return 1;
 }
 
+/**
+ * gst_collect_pads_set_clip_function:
+ * @pads: the collectspads to use
+ * @clipfunc: clip function to install
+ * @user_data: user data to pass to @clip_func
+ *
+ * Install a clipping function that is called right after a buffer is received
+ * on a pad managed by @pads. See #GstCollectDataClipFunction for more info.
+ *
+ * Since: 0.10.26
+ */
+void
+gst_collect_pads_set_clip_function (GstCollectPads * pads,
+    GstCollectPadsClipFunction clipfunc, gpointer user_data)
+{
+  GstCollectPadsPrivate *priv;
+
+  g_return_if_fail (pads != NULL);
+  g_return_if_fail (GST_IS_COLLECT_PADS (pads));
+
+  priv = pads->abidata.ABI.priv;
+
+  priv->clipfunc = clipfunc;
+  priv->clipfunc_user_data = user_data;
+}
+
 /**
  * gst_collect_pads_remove_pad:
  * @pads: the collectspads to use
@@ -506,6 +550,36 @@ gst_collect_pads_collect_range (GstCollectPads * pads, guint64 offset,
   return GST_FLOW_NOT_SUPPORTED;
 }
 
+static gboolean
+gst_collect_pads_is_flushing (GstCollectPads * pads)
+{
+  GSList *walk = NULL;
+  gboolean res = TRUE;
+
+  GST_COLLECT_PADS_PAD_LOCK (pads);
+
+  /* Ensure pads->data state */
+  gst_collect_pads_check_pads_unlocked (pads);
+
+  GST_DEBUG ("Getting flushing state (pads:%p, pads->data:%p)",
+      pads, pads->data);
+
+  for (walk = pads->data; walk; walk = g_slist_next (walk)) {
+    GstCollectData *cdata = walk->data;
+
+    GST_DEBUG_OBJECT (cdata->pad, "flushing:%d", cdata->abidata.ABI.flushing);
+
+    if (cdata->abidata.ABI.flushing) {
+      goto done;
+    }
+  }
+
+  res = FALSE;
+done:
+  GST_COLLECT_PADS_PAD_UNLOCK (pads);
+  return res;
+}
+
 /* FIXME, I think this function is used to work around bad behaviour
  * of elements that add pads to themselves without activating them.
  *
@@ -517,6 +591,8 @@ gst_collect_pads_set_flushing_unlocked (GstCollectPads * pads,
 {
   GSList *walk = NULL;
 
+  GST_DEBUG ("Setting flushing (%d)", flushing);
+
   /* Update the pads flushing flag */
   for (walk = pads->data; walk; walk = g_slist_next (walk)) {
     GstCollectData *cdata = walk->data;
@@ -532,6 +608,10 @@ gst_collect_pads_set_flushing_unlocked (GstCollectPads * pads,
       GST_OBJECT_UNLOCK (cdata->pad);
     }
   }
+  /* Setting the pads to flushing means that we changed the values which
+   * are 'protected' by the cookie. We therefore update it to force a 
+   * recalculation of the current pad status. */
+  pads->abidata.ABI.pad_cookie++;
 }
 
 /**
@@ -554,6 +634,8 @@ gst_collect_pads_set_flushing (GstCollectPads * pads, gboolean flushing)
   g_return_if_fail (GST_IS_COLLECT_PADS (pads));
 
   GST_COLLECT_PADS_PAD_LOCK (pads);
+  /* Ensure pads->data state */
+  gst_collect_pads_check_pads_unlocked (pads);
   gst_collect_pads_set_flushing_unlocked (pads, flushing);
   GST_COLLECT_PADS_PAD_UNLOCK (pads);
 }
@@ -847,13 +929,13 @@ gst_collect_pads_read (GstCollectPads * pads, GstCollectData * data,
  * @data: the data to use
  * @size: the number of bytes to read
  *
- * Get a subbuffer of @size bytes from the given pad @data.
+ * Get a buffer of @size bytes from the given pad @data.
  *
  * This function should be called with @pads LOCK held, such as in the callback.
  *
  * Since: 0.10.18
  *
- * Returns: A sub buffer. The size of the buffer can be less that requested.
+ * Returns: A #GstBuffer. The size of the buffer can be less that requested.
  * A return of NULL signals that the pad is end-of-stream.
  * Unref the buffer after use.
  *
@@ -863,7 +945,7 @@ GstBuffer *
 gst_collect_pads_read_buffer (GstCollectPads * pads, GstCollectData * data,
     guint size)
 {
-  guint readsize;
+  guint readsize, bufsize;
   GstBuffer *buffer;
 
   g_return_val_if_fail (pads != NULL, NULL);
@@ -874,9 +956,14 @@ gst_collect_pads_read_buffer (GstCollectPads * pads, GstCollectData * data,
   if ((buffer = data->buffer) == NULL)
     return NULL;
 
-  readsize = MIN (size, GST_BUFFER_SIZE (buffer) - data->pos);
+  bufsize = GST_BUFFER_SIZE (buffer);
 
-  return gst_buffer_create_sub (buffer, data->pos, readsize);
+  readsize = MIN (size, bufsize - data->pos);
+
+  if (data->pos == 0 && readsize == bufsize)
+    return gst_buffer_ref (buffer);
+  else
+    return gst_buffer_create_sub (buffer, data->pos, readsize);
 }
 
 /**
@@ -885,14 +972,14 @@ gst_collect_pads_read_buffer (GstCollectPads * pads, GstCollectData * data,
  * @data: the data to use
  * @size: the number of bytes to read
  *
- * Get a subbuffer of @size bytes from the given pad @data. Flushes the amount
+ * Get a buffer of @size bytes from the given pad @data. Flushes the amount
  * of read bytes.
  *
  * This function should be called with @pads LOCK held, such as in the callback.
  *
  * Since: 0.10.18
  *
- * Returns: A sub buffer. The size of the buffer can be less that requested.
+ * Returns: A #GstBuffer. The size of the buffer can be less that requested.
  * A return of NULL signals that the pad is end-of-stream.
  * Unref the buffer after use.
  *
@@ -946,6 +1033,8 @@ gst_collect_pads_flush (GstCollectPads * pads, GstCollectData * data,
 
   data->pos += size;
 
+  GST_LOG_OBJECT (pads, "Flushing %d bytes, requested %u", flushsize, size);
+
   if (data->pos >= GST_BUFFER_SIZE (buffer))
     /* _clear will also reset data->pos to 0 */
     gst_collect_pads_clear (pads, data);
@@ -963,10 +1052,10 @@ gst_collect_pads_flush (GstCollectPads * pads, GstCollectData * data,
  * Must be called with LOCK.
  */
 static void
-gst_collect_pads_check_pads (GstCollectPads * pads)
+gst_collect_pads_check_pads_unlocked (GstCollectPads * pads)
 {
-  /* the master list and cookie are protected with the PAD_LOCK */
-  GST_COLLECT_PADS_PAD_LOCK (pads);
+  GST_DEBUG ("stored cookie : %d, used_cookie:%d",
+      pads->abidata.ABI.pad_cookie, pads->cookie);
   if (G_UNLIKELY (pads->abidata.ABI.pad_cookie != pads->cookie)) {
     GSList *collected;
 
@@ -986,10 +1075,13 @@ gst_collect_pads_check_pads (GstCollectPads * pads)
       /* update the stats */
       pads->numpads++;
       data = collected->data;
-      if (data->buffer)
-        pads->queuedpads++;
-      if (data->abidata.ABI.eos)
-        pads->eospads++;
+
+      if (G_LIKELY (!data->abidata.ABI.flushing)) {
+        if (data->buffer)
+          pads->queuedpads++;
+        if (data->abidata.ABI.eos)
+          pads->eospads++;
+      }
 
       /* add to the list of pads to collect */
       ref_data (data);
@@ -998,6 +1090,14 @@ gst_collect_pads_check_pads (GstCollectPads * pads)
     /* and update the cookie */
     pads->cookie = pads->abidata.ABI.pad_cookie;
   }
+}
+
+static inline void
+gst_collect_pads_check_pads (GstCollectPads * pads)
+{
+  /* the master list and cookie are protected with the PAD_LOCK */
+  GST_COLLECT_PADS_PAD_LOCK (pads);
+  gst_collect_pads_check_pads_unlocked (pads);
   GST_COLLECT_PADS_PAD_UNLOCK (pads);
 }
 
@@ -1107,10 +1207,16 @@ gst_collect_pads_event (GstPad * pad, GstEvent * event)
         pads->eospads--;
         data->abidata.ABI.eos = FALSE;
       }
-      GST_OBJECT_UNLOCK (pads);
 
-      /* forward event */
-      goto forward;
+      if (!gst_collect_pads_is_flushing (pads)) {
+        /* forward event if all pads are no longer flushing */
+        GST_DEBUG ("No more pads are flushing, forwarding FLUSH_STOP");
+        GST_OBJECT_UNLOCK (pads);
+        goto forward;
+      }
+      gst_event_unref (event);
+      GST_OBJECT_UNLOCK (pads);
+      goto done;
     }
     case GST_EVENT_EOS:
     {
@@ -1171,6 +1277,8 @@ gst_collect_pads_event (GstPad * pad, GstEvent * event)
   }
 
 forward:
+  GST_DEBUG_OBJECT (pads, "forward unhandled event: %s",
+      GST_EVENT_TYPE_NAME (event));
   res = gst_pad_event_default (pad, event);
 
 done:
@@ -1197,9 +1305,8 @@ gst_collect_pads_chain (GstPad * pad, GstBuffer * buffer)
 {
   GstCollectData *data;
   GstCollectPads *pads;
-  guint64 size;
+  GstCollectPadsPrivate *priv;
   GstFlowReturn ret;
-  GstBuffer **buffer_p;
 
   GST_DEBUG ("Got buffer for pad %s:%s", GST_DEBUG_PAD_NAME (pad));
 
@@ -1212,7 +1319,7 @@ gst_collect_pads_chain (GstPad * pad, GstBuffer * buffer)
   GST_OBJECT_UNLOCK (pad);
 
   pads = data->collect;
-  size = GST_BUFFER_SIZE (buffer);
+  priv = pads->abidata.ABI.priv;
 
   GST_OBJECT_LOCK (pads);
   /* if not started, bail out */
@@ -1225,17 +1332,28 @@ gst_collect_pads_chain (GstPad * pad, GstBuffer * buffer)
   if (G_UNLIKELY (data->abidata.ABI.eos))
     goto unexpected;
 
+  /* see if we need to clip */
+  if (priv->clipfunc) {
+    buffer = priv->clipfunc (pads, data, buffer, priv->clipfunc_user_data);
+
+    if (G_UNLIKELY (buffer == NULL))
+      goto clipped;
+  }
+
   GST_DEBUG ("Queuing buffer %p for pad %s:%s", buffer,
       GST_DEBUG_PAD_NAME (pad));
 
   /* One more pad has data queued */
   pads->queuedpads++;
-  buffer_p = &data->buffer;
-  gst_buffer_replace (buffer_p, buffer);
+  /* take ownership of the buffer */
+  if (data->buffer)
+    gst_buffer_unref (data->buffer);
+  data->buffer = buffer;
+  buffer = NULL;
 
   /* update segment last position if in TIME */
   if (G_LIKELY (data->segment.format == GST_FORMAT_TIME)) {
-    GstClockTime timestamp = GST_BUFFER_TIMESTAMP (buffer);
+    GstClockTime timestamp = GST_BUFFER_TIMESTAMP (data->buffer);
 
     if (GST_CLOCK_TIME_IS_VALID (timestamp))
       gst_segment_set_last_stop (&data->segment, GST_FORMAT_TIME, timestamp);
@@ -1243,6 +1361,7 @@ gst_collect_pads_chain (GstPad * pad, GstBuffer * buffer)
 
   /* While we have data queued on this pad try to collect stuff */
   do {
+    GST_DEBUG ("Pad %s:%s checking", GST_DEBUG_PAD_NAME (pad));
     /* Check if our collected condition is matched and call the collected function
      * if it is */
     ret = gst_collect_pads_check_collected (pads);
@@ -1285,9 +1404,11 @@ gst_collect_pads_chain (GstPad * pad, GstBuffer * buffer)
   while (data->buffer != NULL);
 
 unlock_done:
+  GST_DEBUG ("Pad %s:%s done", GST_DEBUG_PAD_NAME (pad));
   GST_OBJECT_UNLOCK (pads);
   unref_data (data);
-  gst_buffer_unref (buffer);
+  if (buffer)
+    gst_buffer_unref (buffer);
   return ret;
 
 pad_removed:
@@ -1327,6 +1448,12 @@ unexpected:
     ret = GST_FLOW_UNEXPECTED;
     goto unlock_done;
   }
+clipped:
+  {
+    GST_DEBUG ("clipped buffer on pad %s:%s", GST_DEBUG_PAD_NAME (pad));
+    ret = GST_FLOW_OK;
+    goto unlock_done;
+  }
 error:
   {
     /* we print the error, the element should post a reasonable error
