@@ -29,7 +29,7 @@
 #include <stdlib.h>
 #include <math.h>
 
-#include <check.h>
+#include <gst/check/internal-check.h>
 
 #include <gst/gst.h>
 
@@ -108,26 +108,15 @@ gst_check_message_error (msg, GST_MESSAGE_ERROR,		\
  *
  * wrapper for checks END_TEST
  */
-#if CHECK_MAJOR_VERSION > 0 || \
-    (CHECK_MAJOR_VERSION == 0 && CHECK_MINOR_VERSION > 9) || \
-    (CHECK_MAJOR_VERSION == 0 && CHECK_MINOR_VERSION == 9 && CHECK_MICRO_VERSION > 3)
 #define GST_START_TEST(__testname) \
 static void __testname (int __i__)\
 {\
   GST_DEBUG ("test start"); \
   tcase_fn_start (""# __testname, __FILE__, __LINE__);
 
-#define GST_END_TEST END_TEST
-#else
-#define GST_START_TEST(__testname) \
-static void __testname ()\
-{\
-  GST_DEBUG ("test start"); \
-  tcase_fn_start (""# __testname, __FILE__, __LINE__);
-
-#define GST_END_TEST END_TEST
-#endif
-
+#define GST_END_TEST GST_LOG ("cleaning up tasks"); \
+                     gst_task_cleanup_all (); \
+                     END_TEST
 
 /* additional fail macros */
 /**
@@ -264,9 +253,11 @@ MAIN_SYNCHRONIZE();
 G_STMT_START {				\
   _gst_check_threads_running = TRUE;	\
 					\
-  mutex = g_mutex_new ();		\
-  start_cond = g_cond_new ();		\
-  sync_cond = g_cond_new ();		\
+  if (mutex == NULL) {			\
+    mutex = g_mutex_new ();		\
+    start_cond = g_cond_new ();		\
+    sync_cond = g_cond_new ();		\
+  }					\
 } G_STMT_END;
 
 #define MAIN_START_THREAD_FUNCTIONS(count, function, data)	\
@@ -307,6 +298,8 @@ G_STMT_START {							\
   /* join all threads */					\
   GST_DEBUG ("MAIN: joining");					\
   g_list_foreach (thread_list, (GFunc) g_thread_join, NULL);	\
+  g_list_free (thread_list);					\
+  thread_list = NULL;						\
   GST_DEBUG ("MAIN: joined");					\
 } G_STMT_END;
 
@@ -407,8 +400,9 @@ fail_unless (gst_element_set_state (element,			\
 #define GST_CHECK_MAIN(name)					\
 int main (int argc, char **argv)				\
 {								\
-  Suite *s = name ## _suite ();					\
+  Suite *s;                                                     \
   gst_check_init (&argc, &argv);				\
+  s = name ## _suite ();					\
   return gst_check_run_suite (s, # name, __FILE__);		\
 }
 
@@ -417,26 +411,14 @@ int main (int argc, char **argv)				\
 
 gboolean _gst_check_run_test_func (const gchar * func_name);
 
-#if CHECK_MAJOR_VERSION > 0 || \
-    (CHECK_MAJOR_VERSION == 0 && CHECK_MINOR_VERSION > 9) || \
-    (CHECK_MAJOR_VERSION == 0 && CHECK_MINOR_VERSION == 9 && CHECK_MICRO_VERSION > 3)
 static inline void
 __gst_tcase_add_test (TCase * tc, TFun tf, const char * fname, int signal,
-    int start, int end)
+    int allowed_exit_value, int start, int end)
 {
   if (_gst_check_run_test_func (fname)) {
-    _tcase_add_test (tc, tf, fname, signal, start, end);
+    _tcase_add_test (tc, tf, fname, signal, allowed_exit_value, start, end);
   }
 }
-#else
-static inline void
-__gst_tcase_add_test (TCase * tc, TFun tf, const char * fname, int signal)
-{
-  if (_gst_check_run_test_func (fname)) {
-    _tcase_add_test (tc, tf, fname, signal);
-  }
-}
-#endif
 
 #define _tcase_add_test __gst_tcase_add_test
 
